/*********************************************
 * OPL 6.3 Model
 * Author: Emmanuel Rachelson
 * Creation Date: 23 mai 2012 at 08:03:20
 *********************************************/
 
 // Caracteristiques du probleme
 //-----------------------------

 // Horizon de planification
 int NBT=...;				// Nombre de pas de temps	
 range T = 1..NBT;
 
 // type de production
 int NBPROD = ...;
 range PROD = 1..NBPROD;

  // Demande
 float DEM[T,PROD]=...;			// Chroniques de demande (MW)
 
 float MAX_DELTA_PLUS[PROD]=...;		// Limite de surproduction
 float MAX_DELTA_MOINS[PROD]=...;		// Limite de sous-production
 float penalPlus[PROD]=...;			// Pnalits de surproduction
 float penalMoins[PROD]=...;		// Pnalits de sous-production
 
 // Caractristiques des groupes thermiques
 //-----------------------------------------
 
 int NbGT=...;				// Nombre de groupes thermiques
 range GT = 1..NbGT;
 
 // Paliers de production
 int MaxPal[1..3]=...;			// Nombre max de paliers en puissance, primaire et secondaire
 float PalierPui[GT,1..MaxPal[1]]=...;	// Paliers en puissance par groupe thermique
 float PalierPri[GT,1..MaxPal[2]]=...;	// Paliers en primaire par groupe thermique
 float PalierSec[GT,1..MaxPal[3]]=...;	// Paliers en secondaire par groupe thermique

 //float MaxPri[GT] = [maxl(PalierPri[GT,1..MaxPal[2]])];
 float MaxPri[g in GT] = max(p in 1..MaxPal[2]) PalierPri[g,p];
 //float MaxSec[GT] = [maxl(PalierSec[GT,1..MaxPal[3]])];
 float MaxSec[g in GT] = max(p in 1..MaxPal[3]) PalierSec[g,p];
 
 // Dures
 int minMARCHE[GT]=...;	// Dure minimale de marche
 int minPALIER[GT]=...;	// Dure minimale d'un palier

// Gradients 
 float GradPuiDesc[GT]=...;		// Gradient max a la baisse d'un groupe thermique
 float GradPuiMont[GT]=...;		// Gradient max a la hausse d'un groupe thermique
 
 // Cots de fonctionnement
 float coutFixe[GT]=...;		// Cot fixe d'utisation d'un groupe thermique
 float coutProp[GT]=...;		// Cot proportionnel  la puissance
 float coutCgtSSY = ...;		// Cot forfaitaire d'un changement d'etat de services systeme
 float coutModul = ...;			// Cot forfaitaire d'une modulation
 
// Caractristiques des vallees hydrauliques
 //-----------------------------------------

tuple StepUsines{
  int turbinage;
  int pompage;
}  

tuple Index{
  int u;
  int g;
}  

tuple Pdf{
  float turb;
  float pui;
  float pri;
  float sec;
}
 
 int NbUSINES=...;				// Nombre d'usines hydrauliques
 range USI = 1..NbUSINES;

 int NbRESERV=...;				// Nombre de reservoirs
 range RES = 1..NbRESERV;

 int NbSTEP= ...;				// Nombre de STEP
 range STEP= 1..NbSTEP;

 // Reservoirs 
 float valeurEau[RES] = ...;		// coefficient de valorisation des destockes
 float volumeInit[RES] = ...;		// volume initial
 float volumeMin[T,RES] = ...;		// bornes min sur les volumes
 float volumeMax[T,RES] = ...;		// bornes max sur les volumes 
 float apports[T,RES] = ...;		// chronique des apports hydrauliques 

 // Usines
 int Groupe[USI] = ...; 		// nombres de groupes des usines
 float GradDebDesc[USI] = ...;		// gradient max de variation de debit a la baisse
 float GradDebMont[USI] = ...;		// gradient max de variation de debit a la hausse
 StepUsines TabSTEP[STEP]	= ...;	// indices des couple turbine/pompe des STEP 
 {Index} ListeIndex = {<u,g> | u in USI, g in 1..Groupe[u]}; 
 Pdf PdfUSINES[ListeIndex] = ...;	// caracteristique de production des usines

 // Topologie des vallees
 {int} Amont[RES] = ...; 		// indices des usines a l'amont des reservoirs
 {int} Aval [RES] = ...;		// indices des usines a l'aval des reservoirs
 int DelaiAmont[USI] = ...;		// delai de parcours amont des usines
 int DelaiAval [USI] = ...;		// delai de parcours aval des usines

 int coeffDebit = 1800;
 int coeffGrad = 30;

 // Pnalisation des carts hydro
 float penalecart = 0.25;

 // Variables de dcision 
 //-----------------------
 // GROUPES THERMIQUES
 dvar boolean M[T,GT];		// etat de marche du groupe g  la priode t 
 dvar boolean a[T,GT];		// arret du groupe g  la priode t
 dvar boolean d[T,GT];		// demarrage du groupe g  la priode t
 dvar boolean H[T,GT];   	// Palier a pmax
 dvar boolean B[T,GT];   	// Palier a pmin
 dvar boolean I[T,GT];   	// Palier avec primaire
 dvar boolean J[T,GT];   	// Palier avec secondaire
 dvar boolean Mod[T,GT];   	// modulation entre palier
 dvar boolean Css[T,GT];   	// chagnement d'etat de services systeme

 dvar float+ Pui_th[T,GT];
 dvar float+ Pri_th[T,GT];
 dvar float+ Sec_th[T,GT];

 // VALLEES HYDRAULIQUES
 dvar boolean e[T,ListeIndex]; 
 dvar float+ Volume[T,RES];
 dvar float+ ecartplus[T][RES];
 dvar float+ ecartmoins[T][RES];

 dexpr float Debit[t in T][u in USI] = sum(g in 1..Groupe[u]) e[t][<u,g>] * PdfUSINES[<u,g>].turb;
 dexpr float Pui_Hyd[t in T][u in USI] = sum(g in 1..Groupe[u]) e[t][<u,g>] * PdfUSINES[<u,g>].pui;
 dexpr float Pri_Hyd[t in T][u in USI] = sum(g in 1..Groupe[u]) e[t][<u,g>] * PdfUSINES[<u,g>].pri;
 dexpr float Sec_Hyd[t in T][u in USI] = sum(g in 1..Groupe[u]) e[t][<u,g>] * PdfUSINES[<u,g>].sec;

 // PROBLEME GLOBAL
 dvar float+ deltaplus[T,PROD];	// Surproduction  la priode t
 dvar float+ deltamoins[T,PROD];	// Sous-production  la priode t
 
 // A vous de jouer !
 